# matcha

An Idle Inhibitor for Wayland

## Why?

Some Desktop Environments and status bars may not include an idle inhibitor.

Matcha implements a general Wayland idle inhibitor that utilize wayland-protocols as oppossed
to the common approach of using dbus.


## Usage

```
  Usage: matcha [MODE] [OPTION]...
  MODE:
    -d, --daemon     Main instance (Daemon Mode)
    -t, --toggle     Toggle instance (Toggle Mode)
    -s, --status     Get status of inhibitor (Status Mode)"

  Options:
    -o, --off        Start daemon with inhibitor off
    -h, --help       Display this help and exit
```

Matcha consists of two parts, a main instance (via `--daemon`) and a toggle instance (via `--toggle`).

The main instance simply initialize and starts the inhibitor while waiting possible toggle.
The toggle instance, toggles the state and exits (with a possible message).

For example, `matcha` can be used as a part of yambar:

```yml
bar:
  height: 42
  location: top
  background: 3E3F67ff
  font: Ubuntu Nerd Font:pixelsize=16
  right:
    - script:
        path: matcha # Path to matcha executable
        args: ['-d', '--off']
        content:
            - map:
                on-click:
                  left: matcha --toggle
                conditions:
                  inhibit: { string: { text: "🍵" , right-margin: 5} }
                  ~inhibit: { string: { text: "💤", right-margin: 5} }
            - string: { text: "|", right-margin: 5}
```

## Building from Source

### Requirements:

- wayland-client
- meson
- a C11 compiler

```bash
  $ meson setup build --buildtype=release -Dprefix=$INSTALL_LOCATION
  $ meson compile -C build
  $ meson install -C build
```

### Using Nix Flake
```bash
  $ nix build
  $ # Or if you want a static build:
  $ nix build .#static
```
