{
  description = "C Development Environments with LLVM 19 and GCC 14";

  inputs = {
    nixpkgs.url = "github:NixOS/nixpkgs/nixpkgs-unstable";
  };

  outputs = {
    self,
    nixpkgs,
  }: let
    forAllSystems = f:
      nixpkgs.lib.genAttrs nixpkgs.lib.platforms.unix (system:
        f {
          pkgs = import nixpkgs {inherit system;};
        });
  in {
    formatter = forAllSystems ({pkgs, ...}: pkgs.alejandra);

    packages = forAllSystems ({pkgs}: {
      default = pkgs.stdenv.mkDerivation {
        pname = "matcha";
        version = "2.1.0";
        src = ./.;
        nativeBuildInputs = with pkgs; [meson ninja wayland-scanner pkg-config];
        buildInputs = with pkgs; [wayland wayland-protocols];
      };

      static = pkgs.pkgsStatic.stdenv.mkDerivation {
        pname = "matcha";
        version = "2.1.0";
        src = ./.;
        nativeBuildInputs = with pkgs; [meson ninja wayland-scanner pkg-config];
        buildInputs = with pkgs.pkgsStatic; [wayland wayland-protocols];
      };

      makefile = pkgs.stdenv.mkDerivation {
        pname = "matcha";
        version = "2.1.0";
        src = ./.;
        nativeBuildInputs = with pkgs; [wayland-scanner pkg-config];
        buildInputs = with pkgs; [wayland wayland-protocols];
        installPhase = ''
          mkdir -p $out/bin
          cp matcha $out/bin/
        '';
      };

      crossX86 = pkgs.pkgsCross.gnu64.stdenv.mkDerivation {
        pname = "matcha";
        version = "2.1.0";
        src = ./.;
        nativeBuildInputs = with pkgs; [meson ninja wayland-scanner pkg-config];
        buildInputs = with pkgs.pkgsCross.gnu64; [wayland wayland-protocols];
      };

      crossX86Makefile = pkgs.pkgsCross.gnu64.stdenv.mkDerivation {
        pname = "matcha";
        version = "2.1.0";
        src = ./.;
        nativeBuildInputs = with pkgs; [wayland-scanner pkg-config];
        buildInputs = with pkgs.pkgsCross.gnu64; [wayland wayland-protocols];
      };

    });

    devShells = forAllSystems ({pkgs}: let
      commonPackages = with pkgs;
        [
          wayland
          wayland-protocols
          wayland-scanner
          ccache
          cmake
          cmocka
          meson
          ninja
          pkg-config
          python3
        ]
        ++ lib.optionals (!stdenv.isDarwin) [
          valgrind
          gdb
          gf
        ];

      llvmPackages = with pkgs.llvmPackages_19; [
        clang-tools
        clang
        lldb
        llvm
        bintools
      ];

      mkDevShell = {
        stdenv,
        extraPackages ? [],
      }:
        (pkgs.mkShell.override {inherit stdenv;}) {
          packages = commonPackages ++ extraPackages;
        };

      gccStdenv = pkgs.gcc14Stdenv;
      llvmStdenv = pkgs.llvmPackages_17.stdenv;
    in {
      default = mkDevShell {stdenv = gccStdenv;};
      gcc = mkDevShell {stdenv = gccStdenv;};
      gccMold = mkDevShell {
        stdenv = pkgs.stdenvAdapters.useMoldLinker gccStdenv;
        extraPackages = [pkgs.mold];
      };
      gccGold = mkDevShell {
        stdenv = pkgs.stdenvAdapters.useGoldLinker gccStdenv;
      };

      llvm = mkDevShell {
        stdenv = llvmStdenv;
        extraPackages = llvmPackages;
      };
      llvmMold = mkDevShell {
        stdenv = pkgs.stdenvAdapters.useMoldLinker llvmStdenv;
        extraPackages = llvmPackages ++ [pkgs.mold];
      };
      llvmGold = mkDevShell {
        stdenv = pkgs.stdenvAdapters.useGoldLinker llvmStdenv;
        extraPackages = llvmPackages;
      };

      gnu64Cross = pkgs.pkgsCross.gnu64.mkShell {
        nativeBuildInputs = with pkgs; [
          meson
          ninja
          pkg-config
          wayland-scanner
        ];
        buildInputs = with pkgs.pkgsCross.gnu64; [
          wayland
          wayland-protocols
        ];
        packages = with pkgs; [
          pkgs.pkgsCross.gnu64.stdenv.cc
        ];
      };
    });
  };
}
